from typing import Literal, TypeAlias, TypedDict

from . import profile


PointType: TypeAlias = Literal[
    "Comment",
    "CommentWithImage",
    "Status",
    "StatusWithImage",
    "ViewComment",
    "ProfileLogin",
    "ProfileUpdated",
    "ProfileImageUpdated",
    "ViewPage",
    "ViewPageFirstTime",
    "ViewNotification",
    "MessageSent",
    "FeedbackSubmitted",
    "LeadCreated",
    "SessionTracked",
    "InteractiveSessionVote",
    "InteractiveSessionComment",
    "InteractiveSessionQuestion",
    "ManualPoints",
]


class Achievement(TypedDict):
    achievementId: int
    title: str
    message: str
    pointType: PointType
    pointOccurrancesRequired: int
    pointReward: int
    iconUrl: str


class AchievementUnlocked(Achievement):
    unlockedTime: str


class LeaderboardPosition(TypedDict):
    profile: profile.Profile
    position: int
    points: int
    unlockedAchievementsCount: int



from typing import Literal, TypeAlias


ContactInformationKeys: TypeAlias = Literal[
    "phoneNumber",
    "emailAddress",
    "website",
    "address",
    "facebook",
    "twitterHandle",
    "linkedIn",
]


About: TypeAlias = ContactInformationKeys | Literal["subtitle"]


Speaker: TypeAlias = (
    ContactInformationKeys
    | Literal[
        "sortName",
        "companyAndPosition",
        "copy",
    ]
)


ScheduleDay: TypeAlias = Literal["date"]


SessionGroup: TypeAlias = Literal["startTime", "endTime"]


Session: TypeAlias = Literal["startTime", "endTime", "askAQuestionEnabled", "copy"]


SessionSegment: TypeAlias = Literal["startTime", "endTime", "copy"]


Exhibitor: TypeAlias = (
    ContactInformationKeys | Literal["subtitle", "copy", "markerX", "markerY"]
)


GenericGroup: TypeAlias = Literal["cellStyle"]


GenericGroupItem: TypeAlias = (
    ContactInformationKeys | Literal["subtitle", "keywords", "copy"]
)

StringKey: TypeAlias = (
    About
    | Speaker
    | ScheduleDay
    | SessionGroup
    | Session
    | SessionSegment
    | Exhibitor
    | GenericGroup
    | GenericGroupItem
)



from typing import Generator

from entegywrapper.errors import EntegyFailedRequestError
from entegywrapper.schemas.content import Link, TemplateType
from entegywrapper.schemas.profile import Profile


def selected_profile_links(
    self,
    *,
    profile_id: str | None = None,
    external_reference: str | None = None,
    internal_reference: str | None = None,
    badge_reference: str | None = None
) -> Generator[list[Link], None, None]:
    """
    Yields all the profile links the specified profile has in blocks generated
    by the Entegy API's pagination.

    Parameters
    ----------
        `profile_id` (`str`, optional): the profileId of the profile to get; defaults to `None`
        `external_reference` (`str`, optional): the externalReference of the profile to get; defaults to `None`
        `internal_reference` (`str`, optional): the internalReference of the profile to get; defaults to `None`
        `badge_reference` (`str`, optional): the badgeReference of the profile to get; defaults to `None`

    Raises
    ------
        `ValueError`: if no identifier is specified
        `EntegyFailedRequestError`: if the API request fails

    Yields
    -------
        `Generator[list[Link], None, None]`: paginated blocks of selected links
    """
    data = {"pagination": {"start": 0, "limit": 1000}}

    if profile_id is not None:
        data["profileId"] = profile_id
    elif external_reference is not None:
        data["externalReference"] = external_reference
    elif internal_reference is not None:
        data["internalReference"] = internal_reference
    elif badge_reference is not None:
        data["badgeReference"] = badge_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.post(self.api_endpoint + "/v2/ProfileLink/Selected/", data=data)
    yield response["links"]

    while (
        response["pagination"]["start"] + response["pagination"]["limit"]
        < response["pagination"]["count"]
    ):
        data["pagination"]["start"] += data["pagination"]["limit"]

        response = self.post(self.api_endpoint + "/v2/ProfileLink/Selected/", data=data)

        match response["response"]:
            case 200:
                yield response["links"]
            case 401:
                raise EntegyFailedRequestError("Profile not found")
            case _:
                raise EntegyFailedRequestError("Unknown error")


def page_profile_links(
    self,
    template_type: TemplateType,
    *,
    module_id: int | None = None,
    external_reference: str | None = None
) -> Generator[list[Profile], None, None]:
    """
    Yields all the profiles linked to a content page.

    Parameters
    ----------
        `template_type` (`TemplateType`): the templateType of the page
        `module_id` (`int`, optional): the moduleId of the page; defaults to `None`
        `external_reference` (`str`, optional): the externalReference of the page; defaults to `None`

    Raises
    ------
        `ValueError`: if no identifier is specified
        `EntegyFailedRequestError`: if the API request fails

    Yields
    -------
        `Generator[list[Profile], None, None]`: paginated blocks of linked profiles
    """
    data = {"templateType": template_type, "pagination": {"index": 0, "limit": 1000}}

    if module_id is not None:
        data["moduleId"] = module_id
    elif external_reference is not None:
        data["externalReference"] = external_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.post(self.api_endpoint + "/v2/ProfileLink/Page/", data=data)
    yield response["profiles"]

    while (
        response["pagination"]["index"] + response["pagination"]["limit"]
        < response["pagination"]["count"]
    ):
        data["pagination"]["index"] += data["pagination"]["limit"]

        response = self.post(self.api_endpoint + "/v2/ProfileLink/Page/", data=data)

        match response["response"]:
            case 200:
                yield response["profiles"]
            case 401:
                raise EntegyFailedRequestError("Profile not found")
            case 402:
                raise EntegyFailedRequestError("Linked content not found")
            case _:
                raise EntegyFailedRequestError("Unknown error")


def select_profile_link(
    self,
    link: Link,
    *,
    profile_id: str | None = None,
    external_reference: str | None = None,
    internal_reference: str | None = None,
    badge_reference: str | None = None
):
    """
    Selects the specified link for the specified profile.

    Parameters
    ----------
        `link` (`Link`): the link to select
        `profile_id` (`str`, optional): the profileId of the profile; defaults to `None`
        `external_reference` (`str`, optional): the externalReference of the profile; defaults to `None`
        `internal_reference` (`str`, optional): the internalReference of the profile; defaults to `None`
        `badge_reference` (`str`, optional): the badgeReference of the profile; defaults to `None`

    Raises
    ------
        `ValueError`: if no identifier is specified
    """
    data = {"link": link}

    if profile_id is not None:
        data["profileId"] = profile_id
    elif external_reference is not None:
        data["externalReference"] = external_reference
    elif internal_reference is not None:
        data["internalReference"] = internal_reference
    elif badge_reference is not None:
        data["badgeReference"] = badge_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.post(self.api_endpoint + "/v2/ProfileLink/Select/", data=data)

    match response["response"]:
        case 200:
            return
        case 401:
            raise EntegyFailedRequestError("Profile not found")
        case 402:
            raise EntegyFailedRequestError("Linked content not found")
        case _:
            raise EntegyFailedRequestError("Unknown error")


def multi_select_profile_links(self, profiles: list[dict[str, str | list[Link]]]):
    """
    Selects multiple pages on multiple profiles.

    Parameters
    ----------
        `profiles` (`list[dict[str, str | list[Link]]]`): list of profile references with link objects within

    The format of `profiles` is as follows:
    ```python
        [
            {
                "profileId": "ff11c742-346e-4874-9e24-efe6980a7453",
                "links": [
                    {
                        "template_type": "session",
                        "moduleId":1
                    },
                    {
                        "template_type": "session",
                        "moduleId":2
                    },
                    {
                        "template_type": "session",
                        "moduleId":3
                    }
                ]
            },
            ...
        ]
    ```
    """
    data = {"profiles": profiles}

    response = self.post(self.api_endpoint + "/v2/ProfileLink/MultiSelect/", data=data)

    match response["response"]:
        case 401:
            raise EntegyFailedRequestError("Profile doesn't exist")
        case 402:
            raise EntegyFailedRequestError("Invalid templateType")
        case 404:
            raise EntegyFailedRequestError("Invalid moduleId or externalReference")
        case _:
            raise EntegyFailedRequestError("Unknown error")


def deselect_profile_links(
    self,
    link: Link,
    *,
    profile_id: str | None = None,
    external_reference: str | None = None,
    internal_reference: str | None = None,
    badge_reference: str | None = None
):
    """
    Deselects a link for a profile.

    Parameters
    ----------
        `link` (`Link`): the link to deselect
        `profile_id` (`str`, optional): the profileId of the profile; defaults to `None`
        `external_reference` (`str`, optional): the externalReference of the profile; defaults to `None`
        `internal_reference` (`str`, optional): the internalReference of the profile; defaults to `None`
        `badge_reference` (`str`, optional): the badgeReference of the profile; defaults to `None`

    Raises
    ------
        `ValueError`: if no identifier is specified
    """
    data = {"link": link}

    if profile_id is not None:
        data["profileId"] = profile_id
    elif external_reference is not None:
        data["externalReference"] = external_reference
    elif internal_reference is not None:
        data["internalReference"] = internal_reference
    elif badge_reference is not None:
        data["badgeReference"] = badge_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.post(self.api_endpoint + "/v2/ProfileLink/Deselect/", data=data)

    match response["response"]:
        case 401:
            raise EntegyFailedRequestError("Profile doesn't exist")
        case 402:
            raise EntegyFailedRequestError("Linked content not found")
        case _:
            raise EntegyFailedRequestError("Unknown error")


def clear_profile_links(
    self,
    template_type: TemplateType,
    *,
    profile_id: str | None = None,
    external_reference: str | None = None,
    internal_reference: str | None = None,
    badge_reference: str | None = None
):
    """
    Clears all the selected links of a template type on the specified profile.

    Parameters
    ----------
        `template_type` (`TemplateType`): the templateType of links to clear
        `profile_id` (`str`, optional): the profileId of the profile; defaults to `None`
        `external_reference` (`str`, optional): the externalReference of the profile; defaults to `None`
        `internal_reference` (`str`, optional): the internalReference of the profile; defaults to `None`
        `badge_reference` (`str`, optional): the badgeReference of the profile; defaults to `None`

    Raises
    ------
        `ValueError`: if no identifier is specified
    """
    data = {"templateType": template_type}

    if profile_id is not None:
        data["profileId"] = profile_id
    elif external_reference is not None:
        data["externalReference"] = external_reference
    elif internal_reference is not None:
        data["internalReference"] = internal_reference
    elif badge_reference is not None:
        data["badgeReference"] = badge_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.post(self.api_endpoint + "/v2/ProfileLink/Clear/", data=data)

    match response["response"]:
        case 401:
            raise EntegyFailedRequestError("Profile doesn't exist")
        case 402:
            raise EntegyFailedRequestError("Template type doesn't exist")
        case _:
            raise EntegyFailedRequestError("Unknown error")



from typing import Literal, TypeAlias


PageSetting: TypeAlias = Literal[
    "hiddenInApp",
    "unclickable",
    "disableComments",
    "disableRating",
    "requireLogin",
    "reminderAlert",
    "apiManaged",
    "showByProfile",
]



import os
import sys

from entegywrapper.schemas.content import Document, ExternalContent, TemplateType

sys.path.append(os.path.dirname(__file__))


def add_documents(
    self,
    template_type: TemplateType,
    file_documents: list[Document],
    module_id: int | None = None,
    external_reference: str | None = None,
):
    """
    Adds documents to a page.

    Parameters
    ----------
        `template_type` (`TemplateType`): the templateType of the page to add the documents to
        `file_documents` (`list[Document]`): the file documents to add
        `module_id` (`int`): the moduleId of the page to add the documents to
        `external_reference` (`str`): the externalReference of the page to add the documents to

    Raises
    ------
        `ValueError`: if no identifier is specified
        `EntegyFailedRequestError`: if the API request fails
    """
    data = {
        "templateType": template_type,
        "fileDocuments": file_documents,
    }

    if module_id is not None:
        data["moduleId"] = module_id
    elif external_reference is not None:
        data["externalReference"] = external_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.post(self.api_endpoint + "/v2/Document/AddFile", data=data)

    match response["response"]:
        case 200:
            return
        case 400:
            raise ValueError(response["message"])
        case _:
            raise ValueError("Unknown error")


def add_external_content_documents(
    self,
    template_type: TemplateType,
    external_content_items: list[ExternalContent],
    module_id: int | None = None,
    external_reference: str | None = None,
):
    """
    Adds external content documents to a page.

    Parameters
    ----------
        `template_type` (`TemplateType`): the templateType of the page to add the documents to
        `external_content_items` (`list[ExternalContent]`): the external content documents to add
        `module_id` (`int`): the moduleId of the page to add the documents to
        `external_reference` (`str`): the externalReference of the page to add the documents to

    Raises
    ------
        `ValueError`: if no identifier is specified
        `EntegyFailedRequestError`: if the API request fails
    """
    data = {
        "templateType": template_type,
        "externalContentItems": external_content_items,
    }

    if module_id is not None:
        data["moduleId"] = module_id
    elif external_reference is not None:
        data["externalReference"] = external_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.post(
        self.api_endpoint + "/v2/Document/AddExternalContent", data=data
    )

    match response["response"]:
        case 200:
            return
        case 400:
            raise ValueError(response["message"])
        case _:
            raise ValueError("Unknown error")



from entegywrapper.errors import EntegyFailedRequestError
from entegywrapper.schemas.profile import PaymentInfo


def add_profile_payment(
    self,
    payment_info: PaymentInfo,
    *,
    profile_id: str | None = None,
    external_reference: str | None = None,
    internal_reference: str | None = None,
    badge_reference: str | None = None
):
    """
    Adds the given payment info to the specified profile.

    Parameters
    ----------
        `payment_info` (`PaymentInfo`): the payment information to add
        `profile_id` (`str`, optional): the profileId of the profile; defaults to `None`
        `external_reference` (`str`, optional): the externalReference of the profile; defaults to `None`
        `internal_reference` (`str`, optional): the internalReference of the profile; defaults to `None`
        `badge_reference` (`str`, optional): the badgeReference of the profile; defaults to `None`

    Raises
    ------
        `ValueError`: if no identifier is specified
        `EntegyFailedRequestError`: if the API request fails
    """
    data = {"profileId": profile_id}
    data.update(payment_info)

    if profile_id is not None:
        data["profileId"] = profile_id
    elif external_reference is not None:
        data["externalReference"] = external_reference
    elif internal_reference is not None:
        data["internalReference"] = internal_reference
    elif badge_reference is not None:
        data["badgeReference"] = badge_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.post(self.api_endpoint + "/v2/ProfilePayment/Add/", data=data)

    match response["response"]:
        case 200:
            return
        case 400:
            raise EntegyFailedRequestError("Invalid amount")
        case 400:
            raise EntegyFailedRequestError("Invalid currency")
        case 401:
            raise EntegyFailedRequestError("Profile not found")
        case 402:
            raise EntegyFailedRequestError("Duplicate transaction")
        case _:
            raise EntegyFailedRequestError("Unknown error")



from typing import Literal, TypeAlias, TypedDict

from . import project


ProfileExtendedPrivacy: TypeAlias = Literal["Public", "Connections", "Hidden"]


class Permissions(TypedDict, total=False):
    loggedInApp: bool
    loggedInCapture: bool
    showInList: bool
    allowMessaging: bool
    showEmail: ProfileExtendedPrivacy
    showContactNumber: ProfileExtendedPrivacy
    apiManaged: bool
    printedBadge: bool
    optedOutOfEmails: bool
    acceptedTerms: bool


class ProfileReference(TypedDict, total=False):
    profileId: str
    externalReference: str
    internalReference: str
    badgeReference: str
    secondaryId: str


class ProfileRequired(TypedDict):
    type: str
    firstName: str
    lastName: str


class ProfileNotRequired(TypedDict, total=False):
    profileId: str
    externalReference: str
    internalReference: str
    badgeReference: str
    accessCode: str  # ^[A-Za-z0-9]+(?:[._-][A-Za-z0-9]+)*$
    password: str
    title: str
    displayName: str
    organization: str
    position: str
    email: str
    contactNumber: str
    imageUrl: str
    created: str
    lastUpdated: str
    enabled: bool
    permissions: Permissions
    customFields: dict[str, str]
    parentProfile: ProfileReference


class Profile(ProfileRequired, ProfileNotRequired):
    pass


class ProfileType(TypedDict):
    name: str
    isOrganiser: bool
    allowAppLogin: bool
    price: int
    moduleId: int


CustomProfileFieldType: TypeAlias = Literal[
    "MultiChoice",
    "ShortText",
    "MediumText",
    "Facebook",
    "Twitter",
    "Instagram",
    "Website",
]


class MultiChoiceOptions(TypedDict):
    optionId: int
    name: str
    externalMappings: str


class CustomProfileFieldRequired(TypedDict):
    key: str
    name: str
    required: bool
    userAccess: str
    profileVisibility: str
    type: CustomProfileFieldType
    sortOrder: int
    externallyManaged: bool


class CustomProfileFieldNotRequired(TypedDict, total=False):
    options: list[MultiChoiceOptions]


class CustomProfileField(CustomProfileFieldRequired, CustomProfileFieldNotRequired):
    pass


class ProfileCreate(TypedDict):
    externalReference: str
    projectName: str
    projectShortName: str
    eventCode: str
    renewalDate: str
    status: project.ProjectStatus
    type: project.ProjectType
    softwareElements: list[project.SoftwareElement]
    eventInfo: project.ProjectEventInfo


class ProfileUpdate(TypedDict, total=False):
    type: str
    firstName: str
    lastName: str
    externalReference: str
    badgeReference: str
    accessCode: str  # ^[A-Za-z0-9]+(?:[._-][A-Za-z0-9]+)*$
    password: str
    title: str
    organization: str
    position: str
    email: str
    contactNumber: str
    imageUrl: str
    enabled: bool
    permissions: Permissions
    customFields: dict[str, str]


ProfileIdentifier: TypeAlias = Literal[
    "profileId", "externalReference", "internalReference", "badgeReference"
]


PaymentStatus: TypeAlias = Literal["Pending", "Cancelled", "Paid", "Refunded"]


PaymentMethod: TypeAlias = Literal[
    "None", "CreditCard", "DirectDeposit", "Cash", "Cheque", "Other"
]


class PaymentInfoRequired(TypedDict):
    profileId: str
    externalReference: str
    internalReference: str
    badgeReference: str
    currency: str
    amount: int


class PaymentInfoNotRequired(TypedDict, total=False):
    description: str
    amountTax: int
    amountTaxRate: float
    platformFee: int
    platformFeeTax: int
    platformFeeTaxRate: float
    platformFeeInvoiceId: str
    transactionId: str
    gateway: str
    gatewayAccountId: str
    status: PaymentStatus
    method: PaymentMethod


class PaymentInfo(PaymentInfoRequired, PaymentInfoNotRequired):
    pass



from typing import TypedDict

from . import content, profile


class Attendee(TypedDict):
    profile: profile.Profile
    checkInTime: str


class Attended(TypedDict):
    session: content.NamedLink
    checkInTime: str



from entegywrapper.errors import EntegyFailedRequestError


def external_authentication(self, profile_id: str, device_id: str) -> bool:
    """
    Authenticates a user's login via an external system.

    Parameters
    ----------
        `profile_id` (`str`): the profileId that is logging in
        `device_id` (`int`): the deviceId that is logging in

    Returns
    -------
        `bool`: whether this is the user's first login
    """
    data = {"profileId": profile_id, "deviceId": device_id, "requestVersion": 1}

    response = self.post(
        self.api_endpoint + "/v2/Authentication/ExternalProfile", data=data
    )

    match response["response"]:
        case 200:
            return response["firstLogin"]
        case 401:
            raise EntegyFailedRequestError("Profile Doesn't Exist")
        case 406:
            raise EntegyFailedRequestError("Invalid Device Id")
        case 407:
            raise EntegyFailedRequestError("Profile is Banned")
        case 408:
            raise EntegyFailedRequestError("Profile is not Active")
        case _:
            raise EntegyFailedRequestError("Unknown error")



from typing import Literal, TypeAlias, TypedDict

from . import pageSettings, stringKeys


TemplateType: TypeAlias = Literal[
    "Schedule",
    "ScheduleDay",
    "Session",
    "SessionGroup",
    "SessionSegement",
    "Stream",
    "SessionType",
    "Speakers",
    "Speaker",
    "About",
    "Exhibitors",
    "Exhibitor",
    "GenericGroup",
    "GenericGroupPage",
    "FloorPlan",
    "Room",
    "Abstracts",
    "Abstract",
    "HTMLGroup",
    "HTMLPage",
    "Sponsors",
    "Sponsor",
]


class Document(TypedDict):
    name: str
    externalReference: str
    icon: int
    fileUrl: str


class ExternalContent(TypedDict):
    name: str
    externalReference: str
    icon: int
    fileUrl: str
    type: str


class Link(TypedDict):
    templateType: TemplateType
    moduleId: int
    externalReference: str


class NamedLink(Link, TypedDict, total=False):
    name: str


class CategoryRequired(TypedDict):
    moduleId: int
    externalReference: str


class CategoryNotRequired(TypedDict, total=False):
    name: str


class Category(CategoryRequired, CategoryNotRequired):
    pass


class Content(TypedDict):
    contentType: str
    templateType: TemplateType
    moduleId: int
    externalReference: str
    mainImage: str
    strings: dict[stringKeys.StringKey, str]
    pageSettings: dict[pageSettings.PageSetting, bool]
    sortOrder: int


class ContentPage(Content, TypedDict):
    documents: list[Document]
    links: list[Link]
    multiLinks: list[NamedLink]
    selectedCategories: list[Category]


class ContentChild(Content, TypedDict):
    documents: list[Document]
    links: list[Link]
    multiLinks: list[NamedLink]
    selectedCategories: list[Category]


class ContentChildCreateRequired(TypedDict):
    name: str


class ContentChildCreateNotRequired(TypedDict, total=False):
    externalReference: str
    mainImage: str
    strings: dict[stringKeys.StringKey, str]
    links: list[Link]
    sortOrder: int


class ContentChildCreate(ContentChildCreateRequired, ContentChildCreateNotRequired):
    pass


class ContentParent(Content, TypedDict):
    children: list[ContentChild]


ContentIdentifier: TypeAlias = Literal["moduleId", "externalReference"]



import json
import os
import requests
import sys
import time

from requests.structures import CaseInsensitiveDict
from typing import Callable

from entegywrapper.errors import EntegyInvalidAPIKeyError

sys.path.append(os.path.dirname(__file__))

API_ENDPOINTS = {
    "AU": "https://api.entegy.com.au",
    "US": "https://api-us.entegy.com.au",
    "EU": "https://api-eu.entegy.com.au",
}


class EntegyAPI:
    from .Profiles.profiles import (
        all_profiles,
        create_profile,
        get_profile,
        delete_profile,
        update_profile,
        sync_profiles,
        send_welcome_email,
    )
    from .Profiles.profileTypes import (
        get_profile_type,
        create_profile_type,
        update_profile_type,
        delete_profile_type,
        all_profile_types,
    )
    from .Profiles.profileCustom import (
        get_profile_custom,
        create_profile_custom,
        update_profile_custom,
        delete_profile_custom,
        all_profile_custom,
    )
    from .Profiles.profileLinks import (
        selected_profile_links,
        page_profile_links,
        select_profile_link,
        multi_select_profile_links,
        deselect_profile_links,
        clear_profile_links,
    )
    from .Profiles.profilePayments import add_profile_payment
    from .Content.content import (
        get_content,
        get_schedule_content,
        create_content,
        add_children_content,
        update_content,
        delete_content,
    )
    from .Content.categories import (
        available_categories,
        select_categories,
        deselect_categories,
        create_categories,
        create_child_categories,
        update_category,
        delete_categories,
    )
    from .Content.documents import add_documents, add_external_content_documents
    from .Content.multiLink import (
        get_multi_links,
        add_multi_links,
        remove_multi_link,
        remove_all_multi_links,
    )
    from .Points.pointManagement import award_points, get_point_leaderboard, get_points
    from .Plugins.extAuth import external_authentication
    from .Notification.notification import send_notification, send_bulk_notification
    from .AttendanceTracking.attendanceTracking import (
        add_check_in,
        get_attendees,
        get_attended,
    )

    def __init__(
        self,
        api_key: str | list[str],
        api_secret: str | list[str],
        project_id: str,
        region: str = "AU",
    ):
        """
        Creates an Entegy API wrapper to interact with the specified project.

        Parameters
        ----------
            `api_key` (`str | list[str]`): Entegy API key(s)
            `api_secret` (`str | list[str]`): Entegy API secret key(s)
            `project_id` (`str`): Entegy project ID
            `region` (`str`, optional): project region: one of "AU", "US", "EU"; defaults to "AU"
        """
        # if multiple API keys were given, ensure that the number of keys and
        # secrets match
        assert type(api_key) == type(api_secret)
        if type(api_key) == list:
            assert len(api_key) == len(api_secret)
            assert all(type(key) == str for key in api_key)
            assert all(type(secret) == str for secret in api_secret)
        assert type(project_id) == str
        assert type(region) == str
        assert region in API_ENDPOINTS.keys()

        self.api_key = api_key
        self.api_secret = api_secret
        self.current_key_pair = 0
        self.project_id = project_id

        self.headers = CaseInsensitiveDict()
        self.headers["Content-Type"] = "application/json"
        self.get_key()

        self.api_endpoint = API_ENDPOINTS[region]

    def get_key(self) -> str:
        """
        Returns the API Key. If a list of keys was provided, the current key is
        returned.

        Returns
        -------
            `str`: API Key
        """
        if isinstance(self.api_key, list):
            self.headers[
                "Authorization"
            ] = f"ApiKey {self.api_secret[self.current_key_pair]}"
            return self.api_key[self.current_key_pair]

        self.headers["Authorization"] = f"ApiKey {self.api_secret}"
        return self.api_key

    def cycle_key(self):
        """
        Cycles to the next API keypair, wrapping to the first where necessary.
        """
        self.current_key_pair = (self.current_key_pair + 1) % len(self.api_key)

    def get_endpoint(self) -> str:
        """
        Returns the endpoint URL.

        Returns
        -------
            `str`: API endpoint URL
        """
        return self.api_endpoint

    def request(self, method: Callable, endpoint: str, data: dict) -> dict:
        """
        Sends the given data to the given endpoint of the Entegy API, using the given
        method. Internalised to allow for automatic key cycling and error handling.

        Parameters
        ----------
            `method` (`Callable`): method to use to send the request
            `endpoint` (`str`): API endpoint to which to post
            `data` (`dict`): data to post

        Raises
        ------
            `EntegyInvalidAPIKeyError`: if the API keys are invalid

        Returns
        -------
            `dict`: response data
        """
        response = None
        retry_count = 0
        permission_error_count = 0

        data |= {"apiKey": self.get_key(), "projectId": self.project_id}

        while response is None:
            response = method(endpoint, headers=self.headers, data=json.dumps(data))

            # try catch used here as sometimes the response object comes through as
            # a blank JSON object
            try:
                if response.json()["response"] == 403:
                    time.sleep(0.5)
                    permission_error_count += 1
                    if permission_error_count >= 5:
                        raise EntegyInvalidAPIKeyError()
                    response = None
            except:
                response = None
                continue

            assert response is not None

            data = response.json()

            if data["response"] == 489:
                # if there is a rate limit issue, wait the remaining time and try again
                if retry_count >= len(self.api_key):
                    print(
                        f"Rate limit reached, waiting {data['resetDuration']} seconds"
                    )
                    time.sleep(data["resetDuration"] + 2)
                    print("Continuing...")
                    response = None
                else:
                    # update API key
                    self.cycle_key()
                    data["apiKey"] = self.get_key()
                    print(f"Rate limit reached, trying alternate key: {data['apiKey']}")
                    retry_count += 1
                    response = None

        return data

    def post(self, endpoint: str, data: dict) -> dict:
        """
        Posts the given data to the given endpoint of the Entegy API.

        Parameters
        ----------
            `endpoint` (`str`): API endpoint to which to post
            `data` (`dict`): data to post

        Returns
        -------
            `dict`: response data
        """
        return self.request(requests.post, endpoint, data)

    def delete(self, endpoint: str, data: dict) -> dict:
        """
        Deletes the given data from the given endpoint of the Entegy API.

        Parameters
        ----------
            `endpoint` (`str`): API endpoint from which to delete
            `data` (`dict`): data to delete

        Returns
        -------
            `dict`: response data
        """
        return self.request(requests.delete, endpoint, data)



from entegywrapper.errors import EntegyFailedRequestError
from entegywrapper.schemas.profile import CustomProfileField


def get_profile_custom(self, key: str) -> CustomProfileField:
    """
    Returns the custom field specified by the given key.

    Parameters
    ----------
        `key` (`str`): the key of the custom field to return

    Raises
    ------
        `EntegyFailedRequestError`: if the API request fails

    Returns
    -------
        `CustomProfileField`: the custom field specified by the given key
    """
    data = {"key": key}

    response = self.post(self.api_endpoint + "/v2/ProfileCustomField", data=data)

    match response["response"]:
        case 200:
            return response["customField"]
        case 401:
            raise EntegyFailedRequestError("Key doesn't exist")


def create_profile_custom(self, custom_field: CustomProfileField):
    """
    Creates a new custom field for profiles.

    Parameters
    ----------
        `custom_field` (`CustomProfileField`): the custom field to create

    Raises
    ------
        `EntegyFailedRequestError`: if the API request fails
    """
    data = {"customField": custom_field}

    response = self.post(self.api_endpoint + "/v2/ProfileCustomField/Create", data=data)

    match response["response"]:
        case 200:
            return
        case 402:
            raise EntegyFailedRequestError("Malformed field input")
        case 404:
            raise EntegyFailedRequestError("Invalid name")
        case 405:
            raise EntegyFailedRequestError("Key is not unique")
        case 406:
            raise EntegyFailedRequestError("Key is not valid")
        case 407:
            raise EntegyFailedRequestError("Exhausted number of text fields allowed")


def update_profile_custom(self, key: str, custom_field: CustomProfileField):
    """
    Updates the custom profile field specified by the given key with data from
    the given custom field.

    Parameters
    ----------
        `key` (`str`): the key of the custom field to update
        `custom_field` (`CustomProfileField`): the fields to update

    Raises
    ------
        `EntegyFailedRequestError`: if the API request fails
    """
    data = {"key": key, "customField": custom_field}

    response = self.post(self.api_endpoint + "/v2/ProfileCustomField/Update", data=data)

    match response["response"]:
        case 200:
            return
        case 401:
            raise EntegyFailedRequestError("Key doesn't exist")
        case 402:
            raise EntegyFailedRequestError("Invalid name")
        case 404:
            raise EntegyFailedRequestError("Key is not unique")
        case 405:
            raise EntegyFailedRequestError("Key is not valid")
        case 406:
            raise EntegyFailedRequestError("Exhausted number of text fields allowed")
        case _:
            raise EntegyFailedRequestError("Unknown error")


def delete_profile_custom(self, key: str):
    """
    Deletes a custom field.

    Parameters
    ----------
        `key` (`str`): the key of the custom field to delete

    Raises
    ------
        `EntegyFailedRequestError`: if the API request fails
    """
    data = {"key": key}

    response = self.delete(
        self.api_endpoint + "/v2/ProfileCustomField/Delete", data=data
    )

    match response["response"]:
        case 200:
            return
        case 401:
            raise EntegyFailedRequestError("Key doesn't exist")
        case _:
            raise EntegyFailedRequestError("Unknown error")


def all_profile_custom(self) -> list[CustomProfileField]:
    """
    Returns a list all custom fields.

    Returns
    -------
        `list[CustomProfileField]`: all custom fields
    """
    data = {}

    response = self.post(self.api_endpoint + "/v2/ProfileCustomField/All", data=data)

    return response["customFields"]



from entegywrapper.errors import EntegyFailedRequestError
from entegywrapper.schemas.content import Link, NamedLink, TemplateType


def get_multi_links(
    self,
    template_type: TemplateType,
    *,
    module_id: int | None = None,
    external_reference: str | None = None,
) -> list[NamedLink]:
    """
    Returns all the multi links associated with the content page.

    Parameters
    ----------
        `template_type` (`TemplateType`): the templateType of the page
        `module_id` (`int`, optional): the moduleId of the page; defaults to `None`
        `external_reference` (`str`, optional): the externalReference of the page; defaults to `None`

    Raises
    ------
        `ValueError`: if no identifier is specified

    Returns
    -------
        `list[NamedLink]`: all multi links associated with the content page
    """
    data = {"templateType": template_type}

    if module_id is not None:
        data["moduleId"] = module_id
    elif external_reference is not None:
        data["externalReference"] = external_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.post(self.api_endpoint + "/v2/MultiLink", data=data)

    return response["multiLinks"]


def add_multi_links(
    self,
    template_type: TemplateType,
    multi_links: list[Link],
    *,
    module_id: int | None = None,
    external_reference: str | None = None,
):
    """
    Adds multi links to a content page.

    Parameters
    ----------
        `template_type` (`TemplateType`): the templateType of the page to add links to
        `module_id` (`int`): the moduleId of the page to add links to
        `multi_links` (`list[Link]`, optional): the links to add; defaults to `None`
        `external_reference` (`str`, optional): the externalReference of the page; defaults to `None`

    Raises
    ------
        `ValueError`: if no identifier is specified
        `EntegyFailedRequestError`: if the API request fails
    """
    data = {"templateType": template_type, "multiLinks": multi_links}

    if module_id is not None:
        data["moduleId"] = module_id
    elif external_reference is not None:
        data["externalReference"] = external_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.post(self.api_endpoint + "/v2/MultiLink/Add", data=data)

    match response["response"]:
        case 200:
            return
        case 401:
            raise EntegyFailedRequestError("Missing ID")
        case 402:
            raise EntegyFailedRequestError("Page doesn't exist")
        case 404:
            raise EntegyFailedRequestError("Invalid Multi Link")
        case _:
            raise EntegyFailedRequestError("Unknown error")


def remove_multi_link(
    self,
    template_type: TemplateType,
    target_template_type: TemplateType,
    *,
    module_id: int | None = None,
    external_reference: str | None = None,
    target_module_id: int | None = None,
    target_external_reference: str | None = None,
):
    """
    Removes a single multi link from a page.

    Parameters
    ----------
        `template_type` (`TemplateType`): the templateType of the page to remove links from
        `target_template_type` (`TemplateType`): the templateType of the multi link to remove
        `module_id` (`int`, optional): the moduleId of the page to remove links from; defaults to `None`
        `external_reference` (`str`, optional): the externalReference of the page to remove links from; defaults to `None`
        `target_module_id` (`int`, optional): the moduleId of the multi link to remove; defaults to `None`
        `target_external_reference` (`str`, optional): the externalReference of the multi link to remove; defaults to `None`

    Raises
    ------
        `ValueError`: if either no page or no target identifier is specified
        `EntegyFailedRequestError`: if the API request fails
    """
    data = {"templateType": template_type, "targetTemplateType": target_template_type}

    if module_id is not None:
        data["moduleId"] = module_id
    elif external_reference is not None:
        data["externalReference"] = external_reference
    else:
        raise ValueError("Please specify a page identifier")

    if target_external_reference is not None:
        data["targetExternalReference"] = target_external_reference
    elif target_module_id is not None:
        data["targetModuleId"] = target_module_id
    else:
        raise ValueError("Please specify a target identifier")

    response = self.post(self.api_endpoint + "/v2/MultiLink/Remove", data=data)

    match response["response"]:
        case 200:
            return
        case 401:
            raise EntegyFailedRequestError("Missing Id")
        case 402:
            raise EntegyFailedRequestError("Invalid Link")
        case _:
            raise EntegyFailedRequestError("Unknown error")


def remove_all_multi_links(
    self,
    template_type: TemplateType,
    *,
    module_id: int | None = None,
    external_reference: str | None = None,
    link_template_type: TemplateType | None = None,
):
    """
    Removes all the multi links from a page.

    Parameters
    ----------
        `template_type` (`TemplateType`): the templateType of the page
        `module_id` (`int`, optional): the moduleId of the page; defaults to `None`
        `external_reference` (`str`, optional): the externalReference of the page; defaults to `None`
        `link_template_type` (`TemplateType`, optional): the templateType of the page; defaults to `None`

    Raises
    ------
        `ValueError`: if no identifier is specified
    """
    data = {"templateType": template_type}

    if module_id is not None:
        data["moduleId"] = module_id
    elif external_reference is not None:
        data["externalReference"] = external_reference
    else:
        raise ValueError("Please specify an identifier")

    if link_template_type is not None:
        data["linkTemplateType"] = link_template_type

    response = self.post(self.api_endpoint + "/v2/MultiLink/RemoveAll", data=data)

    match response["response"]:
        case 200:
            return
        case 401:
            raise EntegyFailedRequestError("Missing Id")
        case 402:
            raise EntegyFailedRequestError("Invalid linkTemplateType")
        case _:
            raise EntegyFailedRequestError("Unknown error")



from entegywrapper.errors import EntegyFailedRequestError
from entegywrapper.schemas.points import LeaderboardPosition, PointType


def award_points(
    self,
    point_type: PointType,
    points: int,
    *,
    profile_id: str | None = None,
    external_reference: str | None = None,
    internal_reference: str | None = None,
    badge_reference: str | None = None
):
    """
    Awards the given number of points to the specified profile.

    Parameters
    ----------
        `point_type` (`PointType`): the type of points to assign
        `points` (`int`): the amount of points to assign
        `profile_id` (`str`, optional): the profileId for the profile; defaults to `None`
        `external_reference` (`str`, optional): the externalReference of the profile; defaults to `None`
        `internal_reference` (`str`, optional): the internalReference of the profile; defaults to `None`
        `badge_reference` (`str`, optional): the badgeReference of the profile; defaults to `None`

    Raises
    ------
        `ValueError`: if no identifier is specified
    """
    data = {"pointEvent": point_type, "points": points}

    if profile_id is not None:
        data["profileId"] = profile_id
    elif external_reference is not None:
        data["externalReference"] = profile_id
    elif internal_reference is not None:
        data["internalReference"] = profile_id
    elif badge_reference is not None:
        data["badgeReference"] = profile_id
    else:
        raise ValueError("Please specify an identifier")

    response = self.post(self.api_endpoint + "/v2/Point/Award", data=data)

    match response["response"]:
        case 200:
            return
        case 401:
            raise EntegyFailedRequestError("Missing profile reference")
        case 402:
            raise EntegyFailedRequestError("Invalid point event")
        case _:
            raise EntegyFailedRequestError("Unknown error")


def get_points(
    self,
    *,
    profile_id: str | None = None,
    external_reference: str | None = None,
    internal_reference: str | None = None,
    badge_reference: str | None = None
) -> int:
    """
    Returns the amount of points the specified profile has.

    Parameters
    ----------
        `profile_id` (`str`, optional): the profileId for the profile; defaults to `None`
        `external_reference` (`str`, optional): the externalReference of the profile; defaults to `None`
        `internal_reference` (`str`, optional): the internalReference of the profile; defaults to `None`
        `badge_reference` (`str`, optional): the badgeReference of the profile; defaults to `None`

    Raises
    ------
        `ValueError`: if no identifier is specified

    Returns
    -------
        `int`: the amount of points the specified profile has
    """
    data = {}

    if profile_id is not None:
        data["profileId"] = profile_id
    elif external_reference is not None:
        data["externalReference"] = profile_id
    elif internal_reference is not None:
        data["internalReference"] = profile_id
    elif badge_reference is not None:
        data["badgeReference"] = profile_id
    else:
        raise ValueError("Please specify an identifier")

    response = self.post(self.api_endpoint + "/v2/Point/Earned", data=data)

    match response["response"]:
        case 200:
            return response["points"]
        case 401:
            raise EntegyFailedRequestError("Missing profile reference")
        case 402:
            raise EntegyFailedRequestError("Invalid point event")
        case _:
            raise EntegyFailedRequestError("Unknown error")


def get_point_leaderboard(self) -> list[LeaderboardPosition]:
    """
    Returns the leaderboard for the project. The response is sorted by the
    profiles response and includes their position with ties correctly handled.

    Returns
    -------
        `list[LeaderboardPosition]`: the leaderboard position for each profile
    """
    data = {}

    response = self.post(self.api_endpoint + "/v2/Point/Leaderboard", data=data)

    match response["response"]:
        case 200:
            return response["leaderboard"]
        case _:
            raise EntegyFailedRequestError("Unknown error")



from entegywrapper.errors import EntegyFailedRequestError
from entegywrapper.schemas.profile import ProfileIdentifier


def send_notification(
    self,
    title: str,
    message: str,
    *,
    profile_id: str | None = None,
    external_reference: str | None = None,
    internal_reference: str | None = None,
    badge_reference: str | None = None,
    target_page: dict[str, str | int] | None = None
) -> str:
    """
    Sends a notification to the specified profile.

    Parameters
    ----------
        `title` (`str`): the title of the notification
        `message` (`str`): the message of the notification
        `profile_id` (`str`): the profileId of the profile to send the notification to
        `external_reference` (`str`, optional): the externalReference of the profile to send the ; defaults to `None`notification to
        `internal_reference` (`str`, optional): the internalReference of the profile to send the ; defaults to `None`notification to
        `badge_reference` (`str`, optional): the badgeReference of the profile to send the notification to; defaults to `None`
        `target_page` (`dict[str, str | int]`, optional): the page to view when the notification is clicked; defaults to `None`

    The format of `target_page` is as follows:
    ```python
        {
            "templateType": "Exhibitors",
            "moduleId": 1  # could be externalReference instead
        }
    ```

    Raises
    ------
        `ValueError`: if no identifier is specified
        `EntegyFailedRequestError`: if the API request fails

    Returns
    -------
        `str`: API response message, potentially containing failed profile references
    """
    data = {
        "title": title,
        "message": message,
        "alertMessage": "This is an alert message",
    }

    if profile_id is not None:
        data["profileReferences"] = {"profileId": profile_id}
    elif external_reference is not None:
        data["profileReferences"] = {"externalReference": external_reference}
    elif internal_reference is not None:
        data["profileReferences"] = {"internalReference": internal_reference}
    elif badge_reference is not None:
        data["profileReferences"] = {"badgeReference": badge_reference}
    else:
        raise ValueError("Please specify an identifier")

    if target_page is not None:
        data["viewTargetPage"] = target_page

    response = self.post(self.api_endpoint + "/v2/Notification/SendBulk", data=data)

    match response["response"]:
        case 200:
            return ""
        case 201:
            return response["message"]
        case 401:
            raise EntegyFailedRequestError("No profile references specified")
        case 402:
            raise EntegyFailedRequestError("Required string not set")
        case 404:
            raise EntegyFailedRequestError("Target template type doesn't exist")
        case 405:
            raise EntegyFailedRequestError("Target page doesn't exist")
        case _:
            raise EntegyFailedRequestError("Unknown error")


def send_bulk_notification(
    self,
    title: str,
    message: str,
    profile_references: list[dict[ProfileIdentifier, str]],
    *,
    target_page: dict[str, str | int] | None = None
) -> str:
    """
    Sends a notification to the specified profiles.

    Parameters
    ----------
        `title` (`str`): the title of the notification
        `message` (`str`): the message of the notification
        `profile_references` (`list[dict[str, str]]`): the profile references to send the notification to
        `target_page` (`dict[str, str | int]`, optional): the page to view when the notification is clicked; defaults to `None`

    The format of `profile_references` is as follows:
    ```python
        [
            { "profileId": "1234567890" },
            { "externalReference": "1234567890" },
            { "badgeReference": "1234567890" },
            { "internalReference": "1234567890" }
        ]
    ```

    The format of `target_page` is as follows:
    ```python
        {
            "templateType": "Exhibitors",
            "moduleId": 1  # could be externalReference instead
        }
    ```

    Raises
    ------
        `EntegyFailedRequestError`: if the API request fails

    Returns
    -------
        `str`: API response message, potentially containing failed profile references
    """
    data = {
        "profileReferences": profile_references,
        "title": title,
        "message": message,
        "alertMessage": "This is an alert message -- it is not shown anywhere"
        " or documented in the API docs, but it is required.",
    }

    if target_page is not None:
        data["viewTargetPage"] = target_page

    response = self.post(self.api_endpoint + "/v2/Notification/SendBulk", data=data)

    match response["response"]:
        case 200:
            return ""
        case 201:
            return response["message"]
        case 401:
            raise EntegyFailedRequestError("No profile references specified")
        case 402:
            raise EntegyFailedRequestError("Required string not set")
        case 404:
            raise EntegyFailedRequestError("Target template type doesn't exist")
        case 405:
            raise EntegyFailedRequestError("Target page doesn't exist")
        case _:
            raise EntegyFailedRequestError("Unknown error")



from entegywrapper.errors import EntegyFailedRequestError
from entegywrapper.schemas.profile import (
    Profile,
    ProfileCreate,
    ProfileIdentifier,
    ProfileType,
    ProfileUpdate,
)

from typing import Generator


def all_profiles(
    self,
    *,
    include_custom_fields: bool = False,
    include_permissions: bool = False,
    status: str | None = None,
    profile_type: ProfileType | None = None,
    updated_after: str | None = None,
    created_after: str | None = None
) -> Generator[list[Profile], None, None]:
    """
    Yields all user profiles in blocks generated by the Entegy API's pagination.

    Parameters
    ----------
        `include_custom_fields` (`bool`, optional): whether to include custom fields for each profile; defaults to `False`
        `include_permissions` (`bool`, optional): whether to include permissions for each profile; defaults to `False`
        `status` (`str`, optional): only select profiles with this status; defaults to `None`
        `profile_type` (`ProfileType`, optional): only select profiles of this type; defaults to `None`
        `updated_after` (`str`, optional): only select profiles updated after this time; defaults to `None`
        `created_after` (`str`, optional): only select profiles created after this time; defaults to `None`

    Yields
    ------
        `Generator[list[Profile], None, None]`: paginated blocks of user profiles
    """
    data = {
        "pagination": {"start": 0, "limit": 1000},
        "includeCustomFields": include_custom_fields,
        "includePermissions": include_permissions,
    }

    if status is not None:
        data["status"] = status
    if profile_type is not None:
        data["profileType"] = profile_type
    if updated_after is not None:
        data["updatedAfter"] = updated_after
    if created_after is not None:
        data["createdAfter"] = created_after

    response = self.post(self.api_endpoint + "/v2/Profile/All", data=data)
    yield response["profiles"]

    while (
        response["pagination"]["start"] + response["pagination"]["limit"]
        < response["pagination"]["count"]
    ):
        data["pagination"]["start"] += data["pagination"]["limit"]

        response = self.post(self.api_endpoint + "/v2/Profile/All", data=data)

        yield response["profiles"]


def get_profile(
    self,
    *,
    profile_id: str | None = None,
    external_reference: str | None = None,
    internal_reference: str | None = None,
    badge_reference: str | None = None,
    include_custom_fields: bool = False,
    include_permissions: bool = False
) -> Profile:
    """
    Returns the user profile specified by the given identifier

    Parameters
    ----------
        `profile_id` (`str`, optional): the profileId of the profile; defaults to `None`
        `external_reference` (`str`, optional): the externalReference of the profile; defaults to `None`
        `internal_reference` (`str`, optional): the internalReference of the profile; defaults to `None`
        `badge_reference` (`str`, optional): the badgeReference of the profile; defaults to `None`
        `include_custom_fields` (`bool`, optional): whether to include custom fields for each profile; defaults to `False`
        `include_permissions` (`bool`, optional): whether to include permissions for each profile; defaults to `False`

    Raises
    ------
        `ValueError`: if no identifier is specified
        `EntegyFailedRequestError`: if the API request fails

    Returns
    -------
        `Profile`: the user profile specified by the given identifier
    """
    data = {
        "includeCustomFields": include_custom_fields,
        "includePermissions": include_permissions,
    }

    if profile_id is not None:
        data["profileId"] = profile_id
    elif external_reference is not None:
        data["externalReference"] = external_reference
    elif internal_reference is not None:
        data["internalReference"] = internal_reference
    elif badge_reference is not None:
        data["badgeReference"] = badge_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.post(self.api_endpoint + "/v2/Profile/", data=data)

    match response["response"]:
        case 200:
            return response["profile"]
        case 400:
            raise EntegyFailedRequestError("Profile doesn't exist")
        case _:
            raise EntegyFailedRequestError("Unknown error")


def delete_profile(
    self,
    *,
    profile_id: str | None = None,
    external_reference: str | None = None,
    internal_reference: str | None = None,
    badge_reference: str | None = None
):
    """
    Deletes the user profile specified by the given identifier.

    Parameters
    ----------
        `profile_id` (`str`, optional): the profileId of the profile; defaults to `None`
        `external_reference` (`str`, optional): the externalReference of the profile; defaults to `None`
        `internal_reference` (`str`, optional): the internalReference of the profile; defaults to `None`
        `badge_reference` (`str`, optional): the badgeReference of the profile; defaults to `None`

    Raises
    ------
        `ValueError`: if no identifier is specified
        `EntegyFailedRequestError`: if the API request fails
    """
    data = {}

    if profile_id is not None:
        data["profileId"] = profile_id
    elif external_reference is not None:
        data["externalReference"] = external_reference
    elif internal_reference is not None:
        data["internalReference"] = internal_reference
    elif badge_reference is not None:
        data["badgeReference"] = badge_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.delete(self.api_endpoint + "/v2/Profile/Delete", data=data)

    match response["response"]:
        case 200:
            return
        case 401:
            raise EntegyFailedRequestError("No profile found")
        case _:
            raise EntegyFailedRequestError("Unknown error")


def create_profile(self, profile_object: ProfileCreate) -> str:
    """
    Creates the given profile within the Entegy project, returning the newly
    created profileId.

    Parameters
    ----------
        `profile_object` (`ProfileCreate`): a profile object representing the profile to create

    Raises
    ------
        `EntegyFailedRequestError`: if the API request fails

    Returns
    -------
        `str`: profileId of the newly created profile
    """
    data = {"profile": profile_object}

    response = self.post(self.api_endpoint + "/v2/Profile/Create", data=data)

    match response["response"]:
        case 200:
            return response["profileId"]
        case 400:
            raise EntegyFailedRequestError("Invalid custom field key or value")
        case 401:
            raise EntegyFailedRequestError("Missing required Field")
        case 402:
            raise EntegyFailedRequestError("Missing profile type")
        case 404:
            raise EntegyFailedRequestError("Profile has duplicate email")
        case 405:
            raise EntegyFailedRequestError("Badge reference is not unique")
        case 406:
            raise EntegyFailedRequestError("External reference is not unique")
        case 407:
            raise EntegyFailedRequestError("Access code is invalid or not unique")
        case 408:
            raise EntegyFailedRequestError("Project not set to use Profile Passwords")
        case 409:
            raise EntegyFailedRequestError(
                "Profile password doesn't meet password requirements"
            )
        case 410:
            raise EntegyFailedRequestError(
                "Parent profile already has a parent - profile hierarchy is limited to one level"
            )
        case _:
            raise EntegyFailedRequestError("Unknown error")


def update_profile(
    self,
    profile_object: ProfileUpdate,
    *,
    profile_id: str | None = None,
    external_reference: str | None = None,
    internal_reference: str | None = None,
    badge_reference: str | None = None
):
    """
    Updates the user profile specified by the given identifier using the fields
    in the given profile object.

    Parameters
    ----------
        `profile_object` (`ProfileUpdate`): the profile fields to update
        `profile_id` (`str`, optional): the profileId of the profile; defaults to `None`
        `external_reference` (`str`, optional): the externalReference of the profile; defaults to `None`
        `internal_reference` (`str`, optional): the internalReference of the profile; defaults to `None`
        `badge_reference` (`str`, optional): the badgeReference of the profile; defaults to `None`

    Raises
    ------
        `ValueError`: if no identifier is specified
        `EntegyFailedRequestError`: if the API request fails
    """
    data = {"profile": profile_object}

    if profile_id is not None:
        data["profileId"] = profile_id
    elif external_reference is not None:
        data["externalReference"] = external_reference
    elif internal_reference is not None:
        data["internalReference"] = internal_reference
    elif badge_reference is not None:
        data["badgeReference"] = badge_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.post(self.api_endpoint + "/v2/Profile/Update", data=data)

    match response["response"]:
        case 200:
            return
        case 401:
            raise EntegyFailedRequestError("No profile found")
        case 402:
            raise EntegyFailedRequestError("Nothing given to change")
        case 404:
            raise EntegyFailedRequestError("Profile has duplicate email")
        case 405:
            raise EntegyFailedRequestError("Badge reference is not unique")
        case 406:
            raise EntegyFailedRequestError("External reference is not unique")
        case 408:
            raise EntegyFailedRequestError("Project not set to use Profile Passwords")
        case 409:
            raise EntegyFailedRequestError(
                "Profile password doesn't meet password requirements"
            )
        case 410:
            raise EntegyFailedRequestError(
                "Parent profile already has a parent - profile hierarchy is limited to one level"
            )
        case _:
            raise EntegyFailedRequestError("Unknown error")


def sync_profiles(
    self,
    update_reference_type: ProfileIdentifier,
    profiles: list[Profile],
    *,
    group_by_first_profile: bool = False
) -> list[dict[str, str | bool]]:
    """
    Updates or creates profiles in bulk.

    Parameters
    ----------
        `update_reference_type` (`Identifier`): the identifier to use to match profiles for updating
        `profiles` (`list[Profile]`): the list of profiles to create or update
        `group_by_first_profile` (`bool`, optional): whether the parent profile of all profiles in this sync should be set to the first profile in the profiles list (except the first profile itself, which will be set to have no parent); defaults to `False`

    Raises
    ------
        `EntegyFailedRequestError`: if the API request fails

    Returns
    -------
        `list[dict]`: profileIds and `newProfile` flags for each profile in the given list
    """
    data = {
        "updateReferenceType": update_reference_type,
        "profiles": profiles,
        "groupByFirstProfile": group_by_first_profile,
    }

    response = self.post(self.api_endpoint + "/v2/Profile/Sync", data=data)

    match response["response"]:
        case 200:
            return response["results"]
        case 400:
            raise EntegyFailedRequestError("Multiple errors detected")
        case 401:
            raise EntegyFailedRequestError("Nothing given to update")
        case 402:
            raise EntegyFailedRequestError("Exceeded maximum profile update limit")
        case 404:
            raise EntegyFailedRequestError("Invalid updateReferenceType")
        case 500:
            raise EntegyFailedRequestError(
                "An error occurred while syncing the profile"
            )
        case _:
            raise EntegyFailedRequestError("Unknown error")


def send_welcome_email(
    self,
    *,
    profile_id: str | None = None,
    external_reference: str | None = None,
    internal_reference: str | None = None,
    badge_reference: str | None = None
):
    """
    Re-sends the welcome email for a given profile on a given project.

    Parameters
    ----------
        `profile_id` (`str`, optional): the profileId of the profile; defaults to `None`
        `external_reference` (`str`, optional): the externalReference of the profile; defaults to `None`
        `internal_reference` (`str`, optional): the internalReference of the profile; defaults to `None`
        `badge_reference` (`str`, optional): the badgeReference of the profile; defaults to `None`

    Raises
    ------
        `ValueError`: if no identifier is specified
    """
    data = {"profileId": profile_id}

    if profile_id is not None:
        data["profileId"] = profile_id
    elif external_reference is not None:
        data["externalReference"] = external_reference
    elif internal_reference is not None:
        data["internalReference"] = internal_reference
    elif badge_reference is not None:
        data["badgeReference"] = badge_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.post(self.api_endpoint + "/v2/Profile/SendWelcomeEmail", data=data)

    match response["response"]:
        case 200:
            return
        case 401:
            raise EntegyFailedRequestError("No profile found")
        case _:
            raise EntegyFailedRequestError("Unknown error")



from typing import TypedDict

from . import content, profile


class ExhibitorLead(TypedDict):
    Profile: profile.Profile
    scannedTime: str
    syncedTime: str


class ProfileLead(TypedDict):
    exhibitor: content.NamedLink
    scannedTime: str
    syncedTime: str



from entegywrapper.errors import EntegyFailedRequestError
from entegywrapper.schemas.profile import ProfileType


def get_profile_type(
    self, *, name: str | None = None, external_reference: str | None = None
) -> ProfileType:
    """
    Returns the profile type specified by the given identifier.

    Parameters
    ----------
        `name` (`str`, optional): the name of the profile type; defaults to `None`
        `external_reference` (`str`, optional): the externalReference of the profile type; defaults to `None`

    Raises
    ------
        `ValueError`: if no identifier is specified
        `EntegyFailedRequestError`: if the API request fails

    Returns
    -------
        `ProfileType`: the profile type specified by the given identifier
    """
    data = {}

    if name is not None:
        data["name"] = name
    elif external_reference is not None:
        data["externalReference"] = external_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.post(self.api_endpoint + "/v2/ProfileType", data=data)

    match response["response"]:
        case 200:
            return response["profileType"]
        case 401:
            raise EntegyFailedRequestError("Profile type not found")
        case _:
            raise EntegyFailedRequestError("Unknown error")


def create_profile_type(self, profile_type: ProfileType):
    """
    Creates a profile type from the given data.

    Parameters
    ----------
        `profile_type` (`ProfileType`): the data for the profile type to create

    Raises
    ------
        `EntegyFailedRequestError`: if the API request fails
    """
    data = {"profileType": profile_type}

    response = self.post(self.api_endpoint + "/v2/ProfileType/Create", data=data)

    match response["response"]:
        case 200:
            return
        case 402:
            raise EntegyFailedRequestError("Missing Data")
        case 404:
            raise EntegyFailedRequestError("Malformed Data")
        case 405:
            raise EntegyFailedRequestError("Name is empty")
        case 406:
            raise EntegyFailedRequestError("External reference isn't unique")
        case _:
            raise EntegyFailedRequestError("Unknown error")


def update_profile_type(
    self,
    profile_type: ProfileType,
    *,
    name: str | None = None,
    external_reference: str | None = None
):
    """
    Updates the profile type with the data passed in the profileType

    Parameters
    ----------
        `profile_type` (`ProfileType`): the data to update
        `name` (`str`, optional): the name of the profile type; defaults to `None`
        `external_reference` (`str`, optional): the externalReference of the profile type; defaults to `None`

    Raises
    ------
        `ValueError`: if no identifier is specified
        `EntegyFailedRequestError`: if the API request fails
    """
    data = {"profileType": profile_type}

    if name is not None:
        data["name"] = name
    elif external_reference is not None:
        data["externalReference"] = external_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.post(self.api_endpoint + "/v2/ProfileType/Update", data=data)

    match response["response"]:
        case 200:
            return
        case 401:
            raise EntegyFailedRequestError("Profile type not found")
        case 402:
            raise EntegyFailedRequestError("Missing Data")
        case 404:
            raise EntegyFailedRequestError("Malformed Data")
        case 405:
            raise EntegyFailedRequestError("Name is empty")
        case 406:
            raise EntegyFailedRequestError("External reference isn't unique")
        case _:
            raise EntegyFailedRequestError("Unknown error")


def delete_profile_type(
    self, *, name: str | None = None, external_reference: str | None = None
):
    """
    Deletes a profile type. The type cannot be in use.

    Parameters
    ----------
        `name` (`str`, optional): the name of the profile type; defaults to `None`
        `external_reference` (`str`, optional): the externalReference of the profile type; defaults to `None`

    Raises
    ------
        `ValueError`: if no identifier is specified
        `EntegyFailedRequestError`: if the API request fails
    """
    data = {}

    if name is not None:
        data["name"] = name
    elif external_reference is not None:
        data["externalReference"] = external_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.delete(self.api_endpoint + "/v2/ProfileType/Delete", data=data)

    match response["response"]:
        case 200:
            return
        case 401:
            raise EntegyFailedRequestError("Profile type not found")
        case 402:
            raise EntegyFailedRequestError("profile type in use")
        case _:
            raise EntegyFailedRequestError("Unknown error")


def all_profile_types(self) -> list[ProfileType]:
    """
    Returns a list of all profile types.

    Returns
    -------
        `list[ProfileType]`: all profile types
    """
    data = {}

    response = self.post(self.api_endpoint + "/v2/ProfileType/All", data=data)

    return response["profileTypes"]



from entegywrapper.errors import EntegyFailedRequestError
from entegywrapper.schemas.content import Category, TemplateType


def available_categories(
    self,
    template_type: TemplateType,
    *,
    module_id: int | None = None,
    external_reference: str | None = None,
) -> list[Category]:
    """
    Returns the list of available categories for the page specified by the given
    identifier.

    Parameters
    ----------
        `template_type` (`TemplateType`): the templateType of the page
        `module_id` (`int`, optional): the moduleId of the page; defaults to `None`
        `external_reference` (`str`, optional): the externalReference of the page; defaults to `None`

    Raises
    ------
        `ValueError`: if no identifier is specified
        `EntegyFailedRequestError`: if the API request fails

    Returns
    -------
        `list[Category]`: the available categories
    """
    data = {"templateType": template_type}

    if module_id is not None:
        data["moduleId"] = module_id
    elif external_reference is not None:
        data["externalReference"] = external_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.post(self.api_endpoint + "/v2/Categories/Available", data=data)

    match response["response"]:
        case 200:
            return response["availableCategories"]
        case 401:
            raise EntegyFailedRequestError("Missing Id")
        case 402:
            raise EntegyFailedRequestError("templateType doesn't support categories")
        case _:
            raise EntegyFailedRequestError("Unknown error")


def select_categories(
    self,
    template_type: TemplateType,
    categories: list[Category],
    *,
    module_id: int | None = None,
    external_reference: str | None = None,
) -> bool:
    """
    Selects the specified categories for the specified content page.

    Parameters
    ----------
        `template_type` (`TemplateType`): the templateType of the page selecting the categories
        `categories` (`list[Category]`): the categories to select
        `module_id` (`int`, optional): the moduleId of the page selecting the categories; defaults to `None`
        `external_reference` (`str`, optional): the externalReference of the page selecting the categories; defaults to `None`

    Raises
    ------
        `ValueError`: if no identifier is specified
        `EntegyFailedRequestError`: if the API request fails

    Returns
    -------
        `bool`: whether the categories were selected successfully
    """
    data = {"templateType": template_type, "categories": categories}

    if module_id is not None:
        data["moduleId"] = module_id
    elif external_reference is not None:
        data["externalReference"] = external_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.post(self.api_endpoint + "/v2/Categories/Select", data=data)

    match response["response"]:
        case 200:
            return
        case 401:
            raise EntegyFailedRequestError("Missing Id")
        case 402:
            raise EntegyFailedRequestError("templateType doesn't support categories")
        case 404:
            raise EntegyFailedRequestError("No categories")
        case _:
            raise EntegyFailedRequestError("Unknown error")


def deselect_categories(
    self,
    template_type: TemplateType,
    categories: list[Category],
    *,
    module_id: int | None = None,
    external_reference: str | None = None,
) -> bool:
    """
    Deselects the specified categories for the specified content page.

    Parameters
    ----------
        `template_type` (`TemplateType`): the templateType of the page to unselect the categories from
        `categories` (`list[Category]`): the categories to select
        `module_id` (`int`, optional): the moduleId of the page to unselect the categories from; defaults to `None`
        `external_reference` (`str`, optional): the externalReference of the page to unselect the categories from; defaults to `None`

    Raises
    ------
        `ValueError`: if no identifier is specified
        `EntegyFailedRequestError`: if the API request fails

    Returns
    -------
        `bool`: whether the categories were deselected successfully
    """
    data = {"templateType": template_type, "categories": categories}

    if module_id is not None:
        data["moduleId"] = module_id
    elif external_reference is not None:
        data["externalReference"] = external_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.post(self.api_endpoint + "/v2/Categories/Deselect", data=data)

    match response["response"]:
        case 200:
            return
        case 401:
            raise EntegyFailedRequestError("Missing Id")
        case 402:
            raise EntegyFailedRequestError("templateType doesn't support categories")
        case 404:
            raise EntegyFailedRequestError("Category doesn't exist")
        case 405:
            raise EntegyFailedRequestError("Category not selected")
        case 406:
            raise EntegyFailedRequestError("No categories to unselect")
        case _:
            raise EntegyFailedRequestError("Unknown error")


def create_categories(
    self,
    template_type: TemplateType,
    categories: list[Category],
    *,
    module_id: int | None = None,
    external_reference: str | None = None,
):
    """
    Creates categories under a root page.

    Parameters
    ----------
        `template_type` (`TemplateType`): the templateType of the page holding the categories
        `categories` (`list[Category]`): the categories to create
        `module_id` (`int`, optional): the moduleId of the page holding the categories; defaults to `None`
        `external_reference` (`str`, optional): the externalReference of the page holding the categories; defaults to `None`

    Raises
    ------
        `ValueError`: if no identifier is specified
        `EntegyFailedRequestError`: if the API request fails
    """
    data = {"templateType": template_type, "categories": categories}

    if module_id is not None:
        data["moduleId"] = module_id
    elif external_reference is not None:
        data["externalReference"] = external_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.post(self.api_endpoint + "/v2/Categories/Create", data=data)

    match response["response"]:
        case 200:
            return
        case 401:
            raise EntegyFailedRequestError("Missing Id")
        case 402:
            raise EntegyFailedRequestError("Can't create categories here")
        case 404:
            raise EntegyFailedRequestError("Duplicate externalReference")
        case 405:
            raise EntegyFailedRequestError("Missing name")
        case _:
            raise EntegyFailedRequestError("Unknown error")


def create_child_categories(
    self,
    categories: list[Category],
    *,
    module_id: int | None = None,
    external_reference: str | None = None,
):
    """
    Creates categories under another category.

    Parameters
    ----------
        `categories` (`list[Category]`): the categories to create
        `module_id` (`int`): the moduleId of the page holding the categories
        `external_reference` (`int`): the externalReference of the page holding the categories

    Raises
    ------
        `ValueError`: if no identifier is specified
        `EntegyFailedRequestError`: if the API request fails
    """
    data = {"categories": categories}

    if module_id is not None:
        data["moduleId"] = module_id
    elif external_reference is not None:
        data["externalReference"] = external_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.post(self.api_endpoint + "/v2/Categories/CreateChild", data=data)

    match response["response"]:
        case 200:
            return
        case 401:
            raise EntegyFailedRequestError("Missing Id")
        case 404:
            raise EntegyFailedRequestError("Duplicate externalReference")
        case 405:
            raise EntegyFailedRequestError("Missing name")
        case _:
            raise EntegyFailedRequestError("Unknown error")


def update_category(
    self,
    name: str,
    *,
    module_id: int | None = None,
    external_reference: str | None = None,
):
    """
    Changes the name of a category.

    Parameters
    ----------
        `name` (`str`): the new name of the category
        `module_id` (`int`, optional): the moduleId of the category; defaults to `None`
        `external_reference` (`str`, optional): the externalReference of the category; defaults to `None`

    Raises
    ------
        `ValueError`: if no identifier is specified
        `EntegyFailedRequestError`: if the API request fails
    """
    data = {"name": name}

    if module_id is not None:
        data["moduleId"] = module_id
    elif external_reference is not None:
        data["externalReference"] = external_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.post(self.api_endpoint + "/v2/Categories/Update", data=data)

    match response["response"]:
        case 200:
            return
        case 401:
            raise EntegyFailedRequestError("Missing Id")
        case 402:
            raise EntegyFailedRequestError("Missing name")
        case _:
            raise EntegyFailedRequestError("Unknown error")


def delete_categories(
    self,
    template_type: TemplateType,
    categories: list[Category],
    *,
    module_id: int | None = None,
    external_reference: str | None = None,
):
    """
    Creates categories under another category.

    Parameters
    ----------
        `template_type` (`TemplateType`): the templateType of the page
        `categories` (`list[Category]`): the categories to delete
        `module_id` (`int`, optional): the moduleId of the page; defaults to `None`
        `external_reference` (`str`, optional): the externalReference of the page; defaults to `None`

    Raises
    ------
        `ValueError`: if no identifier is specified
        `EntegyFailedRequestError`: if the API request fails
    """
    data = {"templateType": template_type, "categories": categories}

    if module_id is not None:
        data["moduleId"] = module_id
    elif external_reference is not None:
        data["externalReference"] = external_reference
    else:
        raise ValueError("Please specify an identifier")

    response = self.delete(self.api_endpoint + "/v2/Categories/Delete", data=data)

    match response["response"]:
        case 200:
            return
        case 401:
            raise EntegyFailedRequestError("Missing Id")
        case 404:
            raise EntegyFailedRequestError("Category doesn't exist")
        case _:
            raise EntegyFailedRequestError("Unknown error")



from typing import TypedDict

from . import content


class ScheduleDay(content.Content, TypedDict):
    children: list[content.Content]


class Schedule(content.Content, TypedDict):
    days: list[ScheduleDay]


class SessionSegment(content.Content, TypedDict):
    links: list[content.Link]
    multiLinks: list[content.NamedLink]
    documents: list[content.Document]


class Session(content.Content, TypedDict):
    links: list[content.Link]
    multiLinks: list[content.NamedLink]
    documents: list[content.Document]
    selectedCategories: list[content.Category]
    segments: list[SessionSegment]


class SessionGroup(content.Content, TypedDict):
    documents: list[content.Document]
    links: list[content.Link]
    multiLinks: list[content.NamedLink]
    selectedCategories: list[content.Category]



from typing import Literal, TypeAlias, TypedDict


ApiKeyPermission: TypeAlias = Literal[
    "ViewContent",
    "EditContent",
    "EditProfiles",
    "ViewProfiles",
    "Achievements",
    "SendNotifications",
]


Region: TypeAlias = Literal[
    "61a948f2-d505-4b0b-81de-31af6925647e",
    "2b9bd3fc-405e-4df5-888d-f5323e2b5093",
    "86f89b50-1bbb-4019-9ca2-b2d9f4167064",
]


class ProjectEventInfo(TypedDict):
    startDate: str
    endDate: str


ProjectType: TypeAlias = Literal["Event" "Ongoing" "Demo" "Portal", "DemoTemplate"]


ProjectStatus: TypeAlias = Literal[
    "Draft",
    "HandOver",
    "PopulateAndTesting",
    "Production",
    "Finished",
    "Expired",
    "Canceled",
]


SoftwareElement: TypeAlias = Literal[
    "App",
    "StoreListing",
    "Engage",
    "Capture",
    "Track",
    "Interact",
    "Registration",
    "Market",
    "Kiosk",
    "KioskAdditional",
    "EmailDomain",
    "FloorPlan",
]


class Project(TypedDict, total=False):
    projectId: str
    regionId: Region
    regionName: str
    externalReference: str
    internalReference: str
    projectName: str
    projectShortName: str
    iconUrl: str
    eventCode: str
    renewalDate: str
    status: ProjectStatus
    type: ProjectType
    softwareElements: list[SoftwareElement]
    eventInfo: ProjectEventInfo


class ProjectApiKey(TypedDict):
    apiKeyId: str
    description: str
    expireDate: str
    allowedDomains: list[str]
    permissions: list[ApiKeyPermission]
